'use strict';

const DEFAULT_OPTIONS = {
    consoleMethod: 'log',
    consoleObject: 'console'
};
const autoConsole = function autoConsole(options = DEFAULT_OPTIONS) {
    return ({ types: t }) => {
        const replacement = t.memberExpression(t.identifier(options.consoleObject), t.identifier(options.consoleMethod));
        return {
            name: 'auto-console-log',
            visitor: {
                CallExpression(path) {
                    var _a;
                    const { node, parent } = path;
                    if (((_a = node.callee.object) === null || _a === void 0 ? void 0 : _a.name) === options.consoleObject ||
                        t.isIfStatement(parent)) {
                        path.skip();
                        return;
                    }
                    path.replaceWith(t.callExpression(replacement, [node]));
                },
                Identifier(path) {
                    const { node, parent, parentPath } = path;
                    if (t.isSwitchStatement(parent) ||
                        t.isLoop(parentPath.parent) ||
                        t.isIfStatement(parent) ||
                        t.isExpression(parentPath) ||
                        t.isLoop(parent)) {
                        path.skip();
                        return;
                    }
                    if (path.scope.hasBinding(node.name) &&
                        path.isReferencedIdentifier()) {
                        path.replaceWith(t.callExpression(replacement, [node]));
                    }
                },
                BinaryExpression(path) {
                    const { node, parent, parentPath } = path;
                    if (t.isConditionalExpression(parentPath) ||
                        t.isLoop(parentPath) ||
                        t.isConditional(parent) ||
                        t.isSwitchCase(parent)) {
                        path.skip();
                        return;
                    }
                    path.replaceWith(t.callExpression(replacement, [node]));
                },
                ArrowFunctionExpression(path) {
                    path.skip();
                },
                FunctionDeclaration(path) {
                    path.skip();
                },
                VariableDeclarator(path) {
                    path.skip();
                },
                ExpressionStatement(path) {
                    const { parent, parentPath } = path;
                    if (t.isConditional(parentPath.parent) &&
                        !t.isBlockStatement(parent)) {
                        path.skip();
                    }
                },
                LogicalExpression(path) {
                    const { node, parent } = path;
                    if (t.isConditional(parent) || t.isLoop(parent)) {
                        path.skip();
                        return;
                    }
                    path.replaceWith(t.callExpression(replacement, [node]));
                },
                ConditionalExpression(path) {
                    const { node, parent, parentPath } = path;
                    if (t.isConditionalExpression(parentPath) ||
                        t.isLoop(parentPath) ||
                        t.isConditional(parent)) {
                        path.skip();
                        return;
                    }
                    path.replaceWith(t.callExpression(replacement, [node]));
                },
                UpdateExpression(path) {
                    const { node, parent, parentPath } = path;
                    if (t.isConditionalExpression(parentPath) ||
                        t.isLoop(parentPath) ||
                        t.isConditional(parent)) {
                        path.skip();
                        return;
                    }
                    path.replaceWith(t.callExpression(replacement, [node]));
                },
                AssignmentExpression(path) {
                    const { node, parent, parentPath } = path;
                    if (t.isConditionalExpression(parentPath) ||
                        t.isLoop(parentPath) ||
                        t.isConditional(parent)) {
                        path.skip();
                        return;
                    }
                    path.replaceWith(t.callExpression(replacement, [node]));
                },
                UnaryExpression(path) {
                    const { node, parent } = path;
                    if (t.isIfStatement(parent)) {
                        path.skip();
                        return;
                    }
                    path.replaceWith(t.callExpression(replacement, [node]));
                },
                MemberExpression(path) {
                    const { node, parent } = path;
                    if (t.isIfStatement(parent)) {
                        path.skip();
                        return;
                    }
                    path.replaceWith(t.callExpression(replacement, [node]));
                },
                AwaitExpression(path) {
                    const { node } = path;
                    path.replaceWith(t.callExpression(replacement, [node]));
                }
            }
        };
    };
};

module.exports = autoConsole;
